/*
 *  GTK frontend for ray tracer.
 */

#include <unistd.h>
#include <string.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>
#include <pthread.h>
#include "menus.h"
#include "render.h"
#include "t_ray.h"
#include "outputpng.h"
#include "gtk-main.h"

static TRaytracer *R;
static GtkWidget *statusbar;

void main_get_ray_param(int *w, int *h, double *step)
{
  *w = R->getWidth();
  *h = R->getHeight();
  *step = R->getStep();
}

void main_set_ray_param(int w, int h, double step)
{
  R->setWidth(w);
  R->setHeight(h);
  R->setStep(step);
}

void main_save(char *filename)
{
  output_png(filename, R->getPixels(), R->getWidth(), R->getHeight());
}

void main_stop()
{
  /* force quit */
  R->halt();

  /* update status */
  GtkProgress *progress = gnome_appbar_get_progress((GnomeAppBar *) statusbar);
  gnome_appbar_set_status((GnomeAppBar *) statusbar, "  Status: Idle  ");
  gtk_progress_set_percentage(progress, 0);

  /* enable all menus */
  menus_enable();
}

void main_go()
{
  /* disable all menus */
  menus_disable();

  if (R->getStatus() == IDLE)
  {
    R->raytraceScene();
    render_start();
  } 
}

void main_set_filename(char *filename)
{
  int w, h; double step;

  if (R->readScene(filename) == -1)
  {
    /* do something... */
  }
}

int main(int argc, char *argv[])
{
  GtkWidget *gnome_app;
  GtkWidget *draw;
  pthread_t thread;

  int def_height = 100, def_width = 100;

  gnome_init("ray", "2.0", argc, argv);
  gdk_rgb_init();

  gtk_widget_push_visual(gdk_rgb_get_visual());
  gtk_widget_push_colormap(gdk_rgb_get_cmap());

  gnome_app = gnome_app_new("ray", "raytracer");

  draw = gtk_drawing_area_new();
  statusbar = gnome_appbar_new(TRUE, TRUE, GNOME_PREFERENCES_NEVER);
  
  gtk_widget_pop_colormap(); 
  gtk_widget_pop_visual(); 

  gnome_appbar_set_status((GnomeAppBar *) statusbar, "  Status: Idle  ");

  gtk_drawing_area_size(GTK_DRAWING_AREA(draw), def_width, def_height);
   
  add_menus(gnome_app);
  gnome_app_set_statusbar(GNOME_APP(gnome_app), statusbar);
  gnome_app_set_contents(GNOME_APP(gnome_app), draw);

  R = new TRaytracer(def_width, def_height);  
                 
  init_render(draw, statusbar, R);

  gtk_widget_show_all(gnome_app);

  gtk_main();

  return 0;
}

