#include <stdio.h>
#include <iostream.h>
#include "matrix.h"

#define EPSILON 1e-6
#define abs(a) (((a) > 0) ? (a) : (-a))

Matrix::Matrix() { ; }

Matrix::Matrix( int m, int n ) {
  M = new double[ m * n ];
  this->m = m;
  this->n = n;
  identity();
} 

Matrix::~Matrix() {
  delete[] M;
}

Matrix::Matrix( Matrix &src ) {
  if ( this == &src ) return;

  n = src.n;
  m = src.m;
  M = new double[ n * m ];
  memcpy( M, src.M, n * m * sizeof( double )); 
}

void Matrix::operator=( const Matrix &src ) {
  if ( this == &src ) return;

  n = src.n;
  m = src.m;
  M = new double[ n * m ];
  memcpy( M, src.M, n * m * sizeof( double )); 
}

ostream &operator<<( ostream &os, Matrix &r ) {
  r.print( &os );
  return os;
}

Matrix operator*( Matrix &m1, Matrix &m2 ) {
  return( m1.multiply( m2 ));
}

Matrix operator*( Matrix &m1, double k ) {
  return( m1.scale( k ));
}

Matrix operator*=( Matrix &m1, Matrix &m2 ) {
  m1 = m1.multiply( m2 );
  return( m1 );
}

Matrix operator*=( Matrix &m1, double k ) {
  m1 = m1.scale( k );
  return( m1 );
}

Matrix operator+( Matrix &m1, Matrix &m2 ) {
  return( m1.add( m2 ));
}

Matrix operator+=( Matrix &m1, Matrix &m2 ) {
  m1 = m1.add( m2 );
  return( m1 );
}

Matrix operator-( Matrix &m1, Matrix &m2 ) {
  return( m1.sub( m2 ));
}

Matrix operator-=( Matrix &m1, Matrix &m2 ) {
  m1 = m1.sub( m2 );
  return( m1 );
}

int Matrix::isSquare() {
  return( m == n );
}

void Matrix::setRow( int row, double *data ) {
  memcpy( &M[ row ], data, n * sizeof( double ));
}

void Matrix::setCol( int col, double *data ) {
  int i;
  for( i=0; i<n; i++ ) {
    M[ n * i + col ] = *data++;
  }
}

void Matrix::setElements( double *data ) {
  memcpy( M, data, m * n * sizeof( double ));
}

void Matrix::transpose() {
/*
 * Transposes the m x n matrix into an n x m matrix. 
 */
  
  int i, j;
  double *Mt;

  Mt = new double[ n * m ];

  for (i=0; i<m; i++) 
    for (j=0; j<n; j++) 
       Mt[ m*j + i ] = M[ n*i + j ];  

  delete[] M;
  M = Mt;
  swap( n, m ); 
}

Matrix Matrix::add( Matrix &M2 ) {
  int i, j;
  
  Matrix out( m, n );

  if ( M2.m != m && M2.n != n ) {
    return( *this );
  } else {
    for ( i=0; i<m; i++ ) {
      for ( j=0; j<n; j++ ) {
        out.M[ n*i + j ] = M[ n*i + j ] + M2.M[ n*i + j ]; 
      }
    } 
  }
  return( out );
}

Matrix Matrix::sub( Matrix &M2 ) {
  int i, j;
  
  Matrix out( m, n );

  if ( M2.m != m && M2.n != n ) {
    return( *this );
  } else {
    for ( i=0; i<m; i++ ) {
      for ( j=0; j<n; j++ ) {
        out.M[ n*i + j ] = M[ n*i + j ] - M2.M[ n*i + j ]; 
      }
    } 
  }
  return( out );
}

Matrix Matrix::multiply( Matrix &M2 ) {
/*
 * General matrix multiplication routine.  Multiplies m x n matrix M by
 * n x p m2 and returns the m x p result.
 */

  int i, j, k;
  double d;
  int p = M2.n;

  Matrix out( m, p );

  for (i=0; i<p; i++) {           
    for (j=0; j<m; j++) {         
      d = 0.0;
      for (k=0; k<n; k++) 
        d += M2.M[p*k + i] * M[n*j  + k];
      out.M[p*j + i] = d; 
    }
  }
  return out;
}

Matrix Matrix::scale( double k ) {
  int i, j;
  Matrix out( *this );

  for ( i=0; i<m * n; i++ ) {
    out.M[ i ] *= k;
  }
  return out;
}
 
void Matrix::zero() {

  int i,j;

  for (i=0; i<m; i++) 
    for (j=0; j<n; j++) 
       M[n*i + j] = 0.0;
}

void Matrix::identity() {

  int i;
 
  zero(); 
  for (i=0; i<n && i<m; i++) {
    M[n*i + i] = 1.0;
  }
}

void Matrix::exchangeRow ( int m1, int m2 ) {

  int i;
  double temp;
 
  for (i=0; i<n; i++) {
    temp = M[ m1*n + i ];
    M[ m1*n + i ] = M[ m2*n + i ];
    M[ m2*n + i ] = temp;
  } 
}

/*
 *  Prints a matrix, formatted similar to MATLAB's 
 *  output.
 */
#define MAX_STR 20
void Matrix::print( ostream *out ) {

  int i,j;
  double s;
  char *str;

  str = new char[ MAX_STR ]; 

  for (j=0; j<m; j++) { 
    *out << "  ";
    for (i=0; i<n; i++) {
      s = M[ j*n +i ];                
      snprintf( str, MAX_STR, " %10.4f  ",s );
      *out << str; 
    }
    *out << '\n';
  } 
  *out << '\n';
  delete[] str;
}

void Matrix::swap( int &i, int &j ) {
  int temp;

  temp = i;
  i = j;
  j = temp;
}

Matrix Matrix::solve( Matrix &b ) {
/*
 * int Solve( Matrix &b );
 *
 * Solves a system of n equations Ax = b where A is a square matrix
 * containing variable coefficients and b is the right side of the
 * system.
 */
 
  int i, j, k;
  double s;

  Matrix P, L, U, d, c( n, 1 );
  Matrix x( n, 1 );

  decomposeLUP( L, U, P );
  
  d = P * b;
 
 /* forward elimination on L */
  c.zero(); 
  for (k = 0; k<n; k++) {
    s = 0;
    for (j = 0; j<=k-1; j++) {
      s += L.cell( k, j ) * c.M[ j ]; 
    }
    c.M[ k ] = d.M[ k ] - s;
  }
  

 /* back substition with U */
  x.zero();
  for (k = n-1; k >= 0; k--) {
    s = 0;
    for (j = k+1; j<n; j++) 
      s += U.cell( k, j ) * x.M[j];
    x.M[k] = (c.M[k] - s) / U.cell( k, k );  
  }
  return x;
}

void Matrix::decomposeLUP( Matrix &L, Matrix &U, Matrix &P ) {
/*
 * void decomposeLUP( Matrix &L, Matrix &U, Matrix &P );
 *
 * Routine for factoring an NxN matrix M into L and U (diagonal matrices)
 * and a permutation matrix P.
 *
 */

  int i, j, k;
  double max;
  int pivot, pind; 

  if ( !isSquare() ) return;
 /* Make a copy of M so we can muck around with it */  
  Matrix A( *this );

 /* initialize P and LU */
  P = Matrix( n, n );
  L = Matrix( n, n );
  U = Matrix( n, n );  
  U.zero();

  for (k=0; k<n; k++) {
    for (i=k, pivot=0, max=0.0; i<n; i++) {  /* look for largest pivot */    
      if ( A.cell( i, k ) >= EPSILON) {      
        pivot = 1;
        if ( abs( max ) < abs( A.cell( i, k ))) { 
          pind = i;
          max = A.cell( i, k );      
        }
      }
    }
    if (!pivot) {                          /* M's not invertible. */
      cerr << "decomposeLUP: M is singular in column " <<  k << '\n';
      return;
    }
    /* pivot found, exchange row k with row pind */ 
    A.exchangeRow( k, pind );
    P.exchangeRow( k, pind );
    L.exchangeRow( k, pind );

    for (i=k+1; i<n; i++) {
      L.cell( i, k ) = A.cell( i, k ) / A.cell( k, k );
      for (j=k+1; j<n; j++) 
        A.cell( i, j ) = A.cell( i, j ) - L.cell( i, k ) * A.cell( k, j );
    }
    for (i=k; i<n; i++) {
      U.cell( k, i ) = ( abs( A.cell( k, i )) >= EPSILON) ? A.cell( k, i ) : 0; 
    } 
  }
}

