/*
 *  Adds the GTK menus
 */
#include <stdio.h>
#include <gtk/gtk.h>
#include <libgnome/libgnome.h>
#include <libgnomeui/gnome-dialog.h>
#include <libgnomeui/gnome-uidefs.h>
#include <libgnomeui/gnome-app.h>
#include <libgnomeui/gnome-app-helper.h>
#include <libgnomeui/gnome-stock.h>
#include "menus.h"

static void stop_render_cb();
static void exit_cb();
static void open_cb();
static void options_cb();
static void save_cb();
static void render_cb();

static GnomeUIInfo file_menu[] =
{
  GNOMEUIINFO_MENU_OPEN_ITEM(open_cb, NULL),
  GNOMEUIINFO_MENU_SAVE_ITEM(save_cb, NULL),
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_MENU_EXIT_ITEM(exit_cb, NULL),
  GNOMEUIINFO_END
};

static GnomeUIInfo edit_menu[] =
{
  GNOMEUIINFO_MENU_PROPERTIES_ITEM(options_cb, NULL),
  GNOMEUIINFO_SEPARATOR,
  {
    GNOME_APP_UI_ITEM, N_("_Render"),
    N_("Render the scene"),
    render_cb, NULL, NULL, 0, 0, 0, GDK_CONTROL_MASK
  },
  {
    GNOME_APP_UI_ITEM, N_("_Halt Render"),
    N_("Terminate render"),
    stop_render_cb, NULL, NULL, 0, 0, 0, GDK_CONTROL_MASK
  }
};


void add_menus(GtkWidget *app)
{
  GnomeUIInfo main_menu[] =
  {
    GNOMEUIINFO_MENU_FILE_TREE(file_menu),
    GNOMEUIINFO_MENU_EDIT_TREE(edit_menu),
    GNOMEUIINFO_END
  };

  gnome_app_create_menus_with_data(GNOME_APP(app), main_menu, app);  
  gtk_widget_set_sensitive(edit_menu[3].widget, FALSE);
}

static void file_dialog_cb(GtkWidget *widget, gpointer data)
{
  GtkWidget *dialog = (GtkWidget *) data;
  main_set_filename(gtk_file_selection_get_filename(   
                    GTK_FILE_SELECTION(dialog)));

  gtk_widget_destroy(dialog);
}

static void file_save_dialog_cb(GtkWidget *widget, gpointer data)
{
  GtkWidget *dialog = (GtkWidget *) data;
  main_save(gtk_file_selection_get_filename(GTK_FILE_SELECTION(dialog)));
  gtk_widget_destroy(dialog);
}

static void render_cb(GtkWidget *widget, gpointer data)
{
  main_go(); 
}

static void stop_render_cb(GtkWidget *widget, gpointer data)
{
  main_stop();
}

static void options_cb(GtkWidget *widget, gpointer data)
{
  GtkWidget *option_dialog = NULL;
  GtkWidget *wlabel;
  GtkWidget *hlabel;
  GtkObject *wadjust;
  GtkWidget *wspin;
  GtkObject *hadjust;
  GtkWidget *hspin;
  GtkWidget *plabel;
  GtkWidget *pspin;
  GtkObject *padjust;
  GtkWidget *table;
  int result;
  int curw, curh;
  double step;

  main_get_ray_param(&curw, &curh, &step);

  option_dialog = gnome_dialog_new("Preferences", 
                                   "OK",
                                   "Cancel", 
                                   NULL);  
  
  wlabel = gtk_label_new("Width:");
  hlabel = gtk_label_new("Height:");
  plabel = gtk_label_new("Pixel stepsize:");
  
  wadjust = gtk_adjustment_new(curw, 1, 3000, 50, 0, 0);
  wspin = gtk_spin_button_new(GTK_ADJUSTMENT(wadjust), 1.0, 0);
  
  hadjust = gtk_adjustment_new(curh, 1, 3000, 50, 0, 0);
  hspin = gtk_spin_button_new(GTK_ADJUSTMENT(hadjust), 1.0, 0);
  
  padjust = gtk_adjustment_new(step, 1, 4, 1, 0, 0);
  pspin = gtk_spin_button_new(GTK_ADJUSTMENT(padjust), 1.0, 0);
  
  table = gtk_table_new(3, 2, FALSE);
  
  gtk_table_attach(GTK_TABLE(table), wlabel, 0, 1, 0, 1, 0, 0, 10, 0);
  gtk_table_attach(GTK_TABLE(table), wspin, 1, 2, 0, 1, GTK_SHRINK, GTK_SHRINK, 0, 0);
  gtk_table_attach(GTK_TABLE(table), hlabel, 0, 1, 1, 2, 0, 0, 10, 0);
  gtk_table_attach(GTK_TABLE(table), hspin, 1, 2, 1, 2, GTK_SHRINK, GTK_SHRINK, 0, 0);
  gtk_table_attach(GTK_TABLE(table), plabel, 0, 1, 2, 3, 0, 0, 10, 0);
  gtk_table_attach(GTK_TABLE(table), pspin, 1, 2, 2, 3, GTK_SHRINK, GTK_SHRINK, 0, 0);
  
  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(option_dialog)->vbox),
                     table, FALSE, FALSE, 0);

  gnome_dialog_set_close(GNOME_DIALOG(option_dialog), TRUE);
  gnome_dialog_close_hides(GNOME_DIALOG(option_dialog), TRUE);

  gtk_widget_show_all(option_dialog);

  result = gnome_dialog_run(GNOME_DIALOG(option_dialog)); 

  if (result == GNOME_OK)
  {
    main_set_ray_param(gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(wspin)),
                       gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(hspin)),
                       gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(pspin)));
  }

  gtk_widget_destroy(option_dialog);
}
  

static void exit_cb(GtkWidget *widget, gpointer data)
{
  gtk_main_quit();
}

void menus_disable()
{
  /* Disable File->Open */ 
  gtk_widget_set_sensitive(file_menu[0].widget, FALSE);
  /* Disable File->Save */ 
  gtk_widget_set_sensitive(file_menu[1].widget, FALSE);
  /* Disable Edit->Options */ 
  gtk_widget_set_sensitive(edit_menu[0].widget, FALSE);
  /* Disable Edit->Render */ 
  gtk_widget_set_sensitive(edit_menu[2].widget, FALSE);
  /* Enable Edit->Halt */
  gtk_widget_set_sensitive(edit_menu[3].widget, TRUE);
}

void menus_enable()
{
  /* Enable File->Open */ 
  gtk_widget_set_sensitive(file_menu[0].widget, TRUE);
  /* Enable File->Save */ 
  gtk_widget_set_sensitive(file_menu[1].widget, TRUE);
  /* Enable Edit->Options */ 
  gtk_widget_set_sensitive(edit_menu[0].widget, TRUE);
  /* Enable Edit->Render */ 
  gtk_widget_set_sensitive(edit_menu[2].widget, TRUE);
  /* Disable Edit->Halt */
  gtk_widget_set_sensitive(edit_menu[3].widget, FALSE);
}

static void open_cb(GtkWidget *widget, gpointer data)
{
  /* Present the GnomeFile dialog, then return the file pointer */
  static GtkWidget *dialog = NULL;
 
  /* Force only one instance */ 
  if (dialog != NULL)
  {
    gdk_window_show(dialog->window);
    gdk_window_raise(dialog->window);
  }
  else
  {
    dialog = gtk_file_selection_new("Load Scene File");
    gtk_signal_connect(GTK_OBJECT(dialog), "destroy", 
                       GTK_SIGNAL_FUNC(gtk_widget_destroyed),
                       &dialog);

    gtk_signal_connect(GTK_OBJECT (GTK_FILE_SELECTION(dialog)->ok_button), 
                       "clicked", file_dialog_cb, dialog);

    gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(dialog)->cancel_button),
                               "clicked", (GtkSignalFunc) gtk_widget_destroy,
                               GTK_OBJECT(dialog));

    gtk_widget_show_all(dialog);
  }
}

static void save_cb(GtkWidget *widget, gpointer data)
{
  static GtkWidget *dialog = NULL;
 
  /* Force only one instance */ 
  if (dialog != NULL)
  {
    gdk_window_show(dialog->window);
    gdk_window_raise(dialog->window);
  }
  else
  {
    dialog = gtk_file_selection_new("Write PNG File");
    gtk_signal_connect(GTK_OBJECT(dialog), "destroy", 
                       GTK_SIGNAL_FUNC(gtk_widget_destroyed),
                       &dialog);

    gtk_signal_connect(GTK_OBJECT (GTK_FILE_SELECTION(dialog)->ok_button), 
                       "clicked", file_save_dialog_cb, dialog);

    gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(dialog)->cancel_button),
                               "clicked", (GtkSignalFunc) gtk_widget_destroy,
                               GTK_OBJECT(dialog));

    gtk_widget_show_all(dialog);
  }
}
