#include <stdio.h>
#include <stdarg.h>
#include <math.h>
#include "vect.h"
#include "misc.h"
#include "object.h"
#include "poly.h"

Poly::Poly (int n, int *ixarray, double *pointlist) 
{
  Vect v1(3), v2(3);
  int i;
  double x, y, z;

  m_num = n;
  m_points = new int[m_num];
  memcpy(m_points, ixarray, m_num * sizeof (int)); 
 
  m_ptlst = pointlist;

  // calculate plane equation
  v1[0] = pointlist[ixarray[1] * 3] - pointlist[ixarray[0] * 3]; 
  v1[1] = pointlist[ixarray[1] * 3 + 1] - pointlist[ixarray[0] * 3 + 1]; 
  v1[2] = pointlist[ixarray[1] * 3 + 2] - pointlist[ixarray[0] * 3 + 2]; 
  
  v2[0] = pointlist[ixarray[2] * 3] - pointlist[ixarray[0] * 3]; 
  v2[1] = pointlist[ixarray[2] * 3 + 1] - pointlist[ixarray[0] * 3 + 1]; 
  v2[2] = pointlist[ixarray[2] * 3 + 2] - pointlist[ixarray[0] * 3 + 2]; 


  m_normal = v1.cross(v2);
  m_normal.normalize();

  Vect point(3, &m_ptlst[m_points[0]*3]);

  m_d = -(m_normal.dot(point));

  x = ABS(m_normal[0]);  y = ABS(m_normal[1]);  z = ABS(m_normal[2]);

  if (x > y && x > z) m_proj_axis = 0;
  else if (y > x && y > z) m_proj_axis = 1;
  else m_proj_axis = 2;
}

double Poly::intersect (Vect& start, Vect& dir) 
{
  double denom, numer, t;
  int i, j, k=0;
  int a1, a2;
  int pti, ptj;
  double p1x, p2x, p1y, p2y, xint;
    
  numer = m_normal.dot(start) + m_d;  
  denom = m_normal.dot(dir);

  if (denom == 0) return (-1.0); 

  t = - numer/denom;
  if (t <= 0 + EPSILON) return (-1.0);
  
  Vect P = start + t*dir;

  switch (m_proj_axis) 
  {
    case 0:
      a1 = 1; a2 = 2; break;
    case 1:
      a1 = 0; a2 = 2; break;
    case 2:
    default:
      a1 = 0; a2 = 1; break;
  }
  
  // see if point lies in poly

  for (i = 0; i < m_num; i++) 
  {
    pti = m_points[i];
    ptj = m_points[(i+m_num-1) % m_num];
    p1x = m_ptlst[pti*3+a1];
    p1y = m_ptlst[pti*3+a2];
    p2x = m_ptlst[ptj*3+a1];
    p2y = m_ptlst[ptj*3+a2];
    
    if ((P[a2] > MIN (p1y, p2y))  &&
        (P[a2] <= MAX (p1y, p2y)) &&
        (P[a1] <= MAX (p1x, p2x)) &&
        (p1y != p2y))
    {
      xint = (P[a2] - p1y) * (p2x-p1x)/(p2y-p1y)+p1x; 
      if ((p1x == p2x) || (P[a1] <= xint)) 
        k++;
    }
  }          
  if (k % 2) 
  {
    return t; 
  }
  return (-1.0);
}

Vect Poly::getNormal (Vect& location) 
{
  return m_normal;
}
