#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "misc.h"
#include "object.h"
#include "sphere.h"
#include "poly.h"
#include "vect.h"
#include "ray.h"

Raytracer::Raytracer(char *scenefile, int width, int height)
{
  m_render_buf = new char[width * height * 3];

  memset(m_render_buf, 0, width * height * 3);

  m_x = m_y = 0;
  m_width = width;
  m_height = height;

  m_step = 2.0;
  
  readScene(scenefile);
}

Raytracer::Raytracer(int width, int height)
{
  m_render_buf = new char[width * height * 3];

  memset(m_render_buf, 0, width * height * 3);

  m_x = m_y = 0;
  m_width = width;
  m_height = height;

  m_vrp = Vect(3);   
  m_left = Vect(3);
  m_top = Vect(3);
  m_centervp = Vect(3);

  m_num_objs = 0;
  m_num_mats = 0;
  m_num_lights = 0;
  m_step = 2.0;
}

Raytracer::~Raytracer()
{
  delete [] m_render_buf;
}

/*
 *  FIXME: replace with a lex/yacc parser.
 *
 *  This routine should also be moved into a class that
 *  creates an octree or grid containing all objects which
 *  is later fed into the raytracer.
 */
int Raytracer::readScene(char *filename) 
{
  m_vrp = Vect(3);   
  m_left = Vect(3);
  m_top = Vect(3);
  m_centervp = Vect(3);

  m_num_objs = 0;
  m_num_mats = 0;
  m_num_lights = 0;

    #define NUMTAGS 9
    FILE *fp;
    char *Tags[]={"location:","Center:","top:","left:",
                  "Image:","Lights:","Materials:","Objects:","Points:"};
    char Tag[80];
    char ColorStr[8];
    char ch;
    int i;

    char fname[MAXFNAME];     
    int inds[MAXVERTS]; int j; 
    float a, b, c, d;         
    int x,y;

    fp = fopen (filename, "r");
    if (!fp)
      return (-1);
    while (!feof(fp)) {
      fscanf (fp,"%s",Tag);     /* Look for identifier */
      for (i=0; i<NUMTAGS; i++) {
        if (!strcasecmp(Tag,Tags[i])) {
         /* Found an identifier */
         switch (i) {
           case 0:    /* Eye */
             fscanf (fp, "%f%f%f", &a, &b, &c);
             m_vrp[0] = a; m_vrp[1] = b; m_vrp[2] = c;
             break;
           case 1:    /* Center point */
             fscanf (fp, "%f%f%f", &a, &b, &c);
             m_centervp[0] = a; m_centervp[1] = b; m_centervp[2] = c;
             break;
           case 2:    /* top */
             fscanf (fp, "%f%f%f", &a, &b, &c);
             m_top[0]=a; m_top[1]=b; m_top[2]=c;
             break;
           case 3:    /* left */
             fscanf (fp, "%f%f%f", &a, &b, &c);
             m_left[0]=a; m_left[1]=b; m_left[2]=c;
             break;
           case 4:    /* Output Image */
             fscanf (fp,"%s",&fname);
             break;
           case 5:    /* Lights */
             fscanf (fp, "%d", &m_num_lights);  /* Get number of lights */
             i=0;
             while (i<m_num_lights) {
               fscanf (fp, "%79s", Tag);
               if (!strcasecmp(Tag,"Light:")) {
                   fscanf (fp,"%f%f%f",&a, &b, &c);
                   Lights[i].Pos = Vect(3);
                   Lights[i].Pos[0]=a; Lights[i].Pos[1]=b; Lights[i].Pos[2]=c;
                   fscanf (fp,"%8s", ColorStr);
                   convertRGBString (ColorStr, &Lights[i].r, &Lights[i].g, &Lights[i].b);
                   fscanf (fp,"%f", &a);
                   Lights[i].in=a;
                   i++;
               }
               else if (!strcasecmp(Tag,"Lights.")) break;
               if (feof(fp)) return(0);
             }
             break;
           case 8:    /* Points */
             int NumPts;
             fscanf (fp, "%d", &NumPts);  /* Get number of lights */
             i=0;
             while (i<NumPts) {
               fscanf (fp, "%79s", Tag);
               if (!strcasecmp(Tag,"Point:")) {
                 fscanf (fp,"%f%f%f",&a, &b, &c);
                 Verts[3*i] = a;
                 Verts[3*i+1] = b;
                 Verts[3*i+2] = c;
                 i++;
               }
               else if (!strcasecmp(Tag,"Points.")) break;
               if (feof(fp)) return(0);
             }
             break;
           case 6: /* Materials */
             fscanf (fp, "%d", &m_num_mats);
             i=0;
             while (i<m_num_mats) {
               fscanf (fp, "%79s", Tag);
               if (!strcasecmp(Tag, "Material:")) {
                 fscanf (fp, "%8s", ColorStr);
                 convertRGBString (ColorStr, &Materials[i].dr, &Materials[i].dg, 
                           &Materials[i].db);
                 fscanf (fp, "%f%f", &a, &b);
                 Materials[i].ka = a;  Materials[i].kd = b;
                 fscanf (fp, "%8s", ColorStr);
                 convertRGBString (ColorStr, &Materials[i].sr, &Materials[i].sg, 
                           &Materials[i].sb);
                 fscanf (fp, "%f%f%f%f", &a, &b, &c, &d);
                 Materials[i].ks = a; Materials[i].pow = b; 
                 Materials[i].rfl = c; Materials[i].atn = d;
                 fscanf (fp, "%f", &a);
                 Materials[i].ir=a;
                 i++;
               }
               else if (!strcasecmp (Tag, "Materials.")) break;
               if (feof(fp)) return(0);
             }
             break;
           case 7:  /* Objects */
             fscanf (fp, "%d", &m_num_objs);
             i=0 ;
             while (i<m_num_objs) {
               fscanf (fp, "%79s", Tag);
               if (!strcasecmp(Tag, "Sphere:")) {
                 Vect ctr(3);

                 fscanf (fp, "%f%f%f%f%d", &a,&b,&c,&d,&x);
                 ctr[0] = b;
                 ctr[1] = c;
                 ctr[2] = d;
                 ObjectList[i].obj = new Sphere(ctr, a); 
                 ObjectList[i].Material = x;
                 i++;
               }
               if (!strcasecmp (Tag, "Poly:")) {

                 fscanf (fp, "%d", &x);
                 for (j=0; j<x; j++) fscanf (fp, "%d", &inds[j] );
                 ObjectList[i].obj = new Poly(x, inds, Verts);
                 fscanf (fp, "%d", &x);
                 ObjectList[i].Material = x;
                 i++;
               } 
               if (!strcasecmp (Tag, "Objects.")) {
                 if (i<m_num_objs) m_num_objs=i;
                 break;
               }
               if (feof(fp)) return(0);
             }
             break;
         }
        break; 
        }   
      }
    }
    return (0);
} 
           
/* int convertRGBString
 *
 * Converts a string of form "#xxxxxx" to r,g,b floats in range [0..1].
 */
    
int Raytracer::convertRGBString (char *Color, float *r, float *g, float *b) 
{
    int x, y, z;

    if (Color[0]!='#') return (-1);
    else {
        sscanf (++Color, "%2x",&x);
        Color+=2;
        sscanf (Color, "%2x",&y);
        Color+=2;
        sscanf (Color, "%2x",&z);
        *r = (float) x / 255.0;
        *g = (float) y / 255.0;
        *b = (float) z / 255.0;
    }
    return (0);
}
       
/*
 *  Main entry point into raytracer.  Finds width*height pixels on the 
 *  viewplane and traces rays through them from the eye to determine the 
 *  color for those pixels.
 */
void Raytracer::raytraceScene() 
{
  Vect ray = Vect(3);
  Vect dir = Vect(3);
  float pixel[3];
  float sample[3];
  int i, vstep, ustep;
  char *scene;

  scene = m_render_buf;

  /* 
   *  U and V are step sizes in X and Y respectively.
   *  They are 1/m_step of a pixel step; m_step**2 
   *  jittered rays are cast for each pixel and averaged
   *  at the end.
   */
  Vect u = ((2.0 * (m_centervp - m_left)) / m_width);
  Vect v = ((2.0 * (m_centervp - m_top)) / m_height);
  ray = m_left + m_top - m_centervp;

 /*
  * Main ray tracing loop.  This code finds a destination pixel on the 
  * viewplane by iterating in X, Y and Z first in m_height and then by m_width.
  */

  for (m_y = 0; m_y < m_height; m_y++) 
  {

    /* save start of scanline */
    Vect temp = ray;

    for (m_x = 0; m_x < m_width; m_x++) 
    {
      sample[0] = 0.0;
      sample[1] = 0.0;
      sample[2] = 0.0;

      /* find four subpixels in v */
      for (vstep = 0; vstep < m_step; vstep++)
      {
        /* find four subpixels in u */
        for (ustep = 0; ustep < m_step; ustep++)
        {
          ray += u/m_step; 

          /* add jitter */     
          double ju = rand()/(RAND_MAX+1.0) * (u/m_step).getLength();
          double jv = rand()/(RAND_MAX+1.0) * (v/m_step).getLength();
 
          dir = ray - m_vrp + ju * u + jv * v;
          dir.normalize();

          pixel[0] = 0.0;
          pixel[1] = 0.0;
          pixel[2] = 0.0;

          getPixelColor (m_vrp, dir, 1.0, pixel, pixel+1, pixel+2, 1);

          sample[0] += pixel[0];
          sample[1] += pixel[1];
          sample[2] += pixel[2];
        }
        /* return to start of block for next row */
        ray -= u;
        ray += v/m_step;
      }
      /* advance one full pixel to m_left */
      ray -= v;
      ray += u;

      for (i=0; i<3; i++) 
      { 
        *scene++ = (int) (255.0 * (sample[i] / (m_step * m_step))); 
      }
    }
    /* restore scanline and increment in v */
    ray = temp + v;
  }
}


/* getPixelColor
 *
 * Finds the color at the given WC point by determining intersections 
 * with objects.
 */
void Raytracer::getPixelColor (Vect &start, Vect &ray, double ir, float *r, 
                               float *g, float *b, int depth) 
{

  int i;
  double t, tnew; 
  int ObjInter=-1;

  t=MAX_DBL;
 
  for (i=0; i< m_num_objs;i++) 
  {
    tnew = ObjectList[i].obj->intersect(start, ray);
    if (tnew < t && tnew != -1.0) {
      ObjInter = i;
      t = tnew;
    } 
  }
  /* Shade the pixel */
  if (ObjInter!=-1) shade(ObjInter, start, ray, t, ir, r, g, b, depth);
 
} 

/* 
 * Determines the shade at a pixel by evaluating the illumination 
 * equation at the point of intersection of a ray and an object.
 */
void Raytracer::shade (int objectindex, Vect& startray, Vect& v, double t,
                       double ir, float *r, float *g, float *b, int depth) 
{
  int MatIndex;
  int i,j;
  float dotNL, dotRV;
  float S;
  float rColor[3];
  float Scale;
  float fatt = 1.0;
  double t2, tlight;
  int is_exiting = 0;

  /* Do ambient term */
  MatIndex = ObjectList[objectindex].Material;
  *r = Materials[MatIndex].dr*Materials[MatIndex].ka;
  *g = Materials[MatIndex].dg*Materials[MatIndex].ka;
  *b = Materials[MatIndex].db*Materials[MatIndex].ka;

  /* Get Intersection point */
  Vect intersect = t * v + startray;

  /* Flip V so it points towards viewer */
  v = -1.0 * v;            

  /* Get surface normal */
  Vect N = ObjectList[objectindex].obj->getNormal (intersect); 


  /* If inside a sphere, flip the normal */
  is_exiting = (N.dot(v) < 0);
  if (is_exiting && typeid(*ObjectList[objectindex].obj) == typeid(Sphere))
  {
    N = -1.0 * N;
  }

  /* Find diffuse / specular contributions. */
  for (i=0; i<m_num_lights;i++) 
  {
     /* L = ray to light (shadow ray) */
     Vect L = Lights[i].Pos - intersect;
     tlight = L.getLength();

     if (tlight > 0) 
       L /= tlight;

     /* Fire shadow ray at objects */
     S = 1.0;
     for (j=0; j<m_num_objs; j++) {
       t2 = ObjectList[j].obj->intersect (intersect, L); 
       if (t2 != -1.0 && t2 < tlight) S=0.0;         
     }

     Scale = S*Lights[i].in*fatt;
     
     dotNL = N.dot(L);
     if (dotNL > 0) {
       /* diffuse component */
       *r += Scale * dotNL*Materials[MatIndex].kd*Materials[MatIndex].dr;
       *g += Scale * dotNL*Materials[MatIndex].kd*Materials[MatIndex].dg;
       *b += Scale * dotNL*Materials[MatIndex].kd*Materials[MatIndex].db;
     }
    
     Vect R = ((2.0 * dotNL * N) - L);
     dotRV = R.dot(v);
     if (dotRV > 0) {
       /* specular component */
       Scale *= pow (dotRV, Materials[MatIndex].pow)*Materials[MatIndex].ks;
       *r += Scale*Materials[MatIndex].sr;
       *g += Scale*Materials[MatIndex].sg;
       *b += Scale*Materials[MatIndex].sb;
     }
  }
  if (depth < MAXDEPTH) 
  {
    /* find Reflection contribution */
    if (Materials[MatIndex].rfl > 0.0) 
    {

      Vect R = (2.0 * N.dot(v) * N) - v;

      for (j=0;j<3;j++) rColor[j] = 0.0;
      getPixelColor (intersect, R, ir, &rColor[0], &rColor[1], &rColor[2], depth+1);
      *r+=Materials[MatIndex].ks*rColor[0];
      *g+=Materials[MatIndex].ks*rColor[1];
      *b+=Materials[MatIndex].ks*rColor[2];
    }
    /* find transmission contribution */
    if (Materials[MatIndex].atn > 0.0 && Materials[MatIndex].ir > 0.0) 
    {
      double nu_t;
      double nu_r;
      double descriminant;
      double n_dot_v;

      /* FIXME: doesn't handle object in an object */
      nu_t = (is_exiting) ? 1.0 : Materials[MatIndex].ir;
      nu_r = ir / nu_t;
      n_dot_v = N.dot(v);

      /* For some reason, using pow() here has troubles */
      descriminant = 1 - (nu_r * nu_r) * (1 - (n_dot_v * n_dot_v));

      if (descriminant >= 0)   /* no total internal reflection */
      {
        Vect T = -nu_r * v + (nu_r * n_dot_v - sqrt(descriminant)) * N; 

        for (j=0;j<3;j++) rColor[j] = 0.0; 

        getPixelColor (intersect, T, nu_t, &rColor[0], &rColor[1], &rColor[2], depth+1);
        *r+=Materials[MatIndex].atn*rColor[0];
        *g+=Materials[MatIndex].atn*rColor[1];
        *b+=Materials[MatIndex].atn*rColor[2];
      }
    }
  }
  *r = (*r > 1.0) ? 1.0 : *r;
  *g = (*g > 1.0) ? 1.0 : *g;
  *b = (*b > 1.0) ? 1.0 : *b;
}

char *Raytracer::getPixels()
{
  return m_render_buf;
}

int Raytracer::getWidth()
{
  return m_width;
}

int Raytracer::getHeight()
{
  return m_height;
}

double Raytracer::getStep()
{
  return m_step;
}

