#ifndef RAY_HEADER
#define RAY_HEADER

#include "object.h"
#include "vect.h"
#include "misc.h"

class Raytracer
{
  /* File variables */
  Vect m_vrp; 
  Vect m_left;
  Vect m_top;
  Vect m_centervp;

  int m_num_objs;                  

  struct items {
    int Material;
    Object *obj;
  } ObjectList[MAXOBJS];

  int m_num_mats;                 /* Total number of materials */
  struct Mat {                    /* List of all materials in the scene */
    float dr, dg, db;
    float ka, kd;
    float sr, sg, sb;
    float ks, pow, rfl, atn, ir;
  } Materials[MAXMATS];

  struct Light {                 /* All information regarding scene lighting */
    Vect Pos;
    float r, g, b;
    float in;
  } Lights[MAXLIGHTS];

  double Verts[MAXPOINTS * 3];   /* The pointlist */

  int m_num_lights;              /* Total number of lights */

 
  int convertRGBString(char* color, float* r, float* g, float* b);

  void getPixelColor(Vect& start, Vect& direction, double ir,
                     float* r, float* g, float* b, int depth);

  void shade(int objectindex, Vect& start, Vect& direction,
             double t, double ir, float* r, float* g,
             float* b, int depth);

  
protected:       
  char *m_render_buf;
  int m_x;
  int m_y;
  int m_width;
  int m_height;
  double m_step;

public:
  Raytracer(char *scenefile, int width, int height);
  Raytracer(int width, int height);
  ~Raytracer();
  virtual void raytraceScene();
  char *getPixels();
  int getWidth();
  int getHeight();
  double getStep();
  int readScene(char *filename);
};

#endif
