/*
 *  This handles all of the nasty routines for rendering.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <libgnomeui/gnome-appbar.h>
#include "gtk-main.h"
#include "stretch.h"
#include "t_ray.h"
#include "render.h"

typedef struct _timerparam
{
  GtkWidget *widget;
  GtkWidget *appbar;
  TRaytracer *ray;
} timerparam;

static guint32 save_width = 0;
static guint32 save_height = 0;

static char *copy_buffer = 0;
static int posx, posy;  /* The next pixel to be written */

static timerparam tparam;

gboolean render_expose_cb(GtkWidget *widget, GdkEventExpose *event,
                          TRaytracer *ray)
{
  render_screen(widget, ray); 
}

gint render_time_cb(timerparam *t)
{
  double pctdone;
  gboolean done = (t->ray->getStatus() == IDLE);

  render_screen(t->widget, t->ray);

  /* update the progress bar */
  GtkProgress *progress = gnome_appbar_get_progress((GnomeAppBar *) t->appbar);
  t->ray->getProgress(pctdone);

  if (done)
  {
    /* let everyone else know we're done. */
    main_stop();

    /* unload timer interrupt */
    return FALSE;
  }
  else
  {
    gtk_progress_set_percentage(progress, (gfloat) pctdone);
    return TRUE;
  }
}

void init_render(GtkWidget *widget, GtkWidget *appbar, TRaytracer *ray)
{
  posx = posy = 0;
  save_width = 0;
  save_height = 0;
  copy_buffer = NULL;

  tparam.widget = widget;
  tparam.ray = ray;
  tparam.appbar = appbar;

  gtk_signal_connect(GTK_OBJECT(widget), "expose-event", 
                     GTK_SIGNAL_FUNC(render_expose_cb), 
                     (gpointer) ray);
}

/*
 *  Starts the periodic interrupt.
 */
void render_start()
{
  char status[80];

  posx = posy = 0;
  save_width = 0;
  save_height = 0;
  
  /* set status bar to say we're rendering */
  snprintf(status, 80, "  Status: Rendering (%dx%d)  ", 
           tparam.ray->getWidth(), tparam.ray->getHeight());
 
  gnome_appbar_set_status((GnomeAppBar *)tparam.appbar, status);

  /* connect to a timeout */
  gtk_timeout_add(750, (GtkFunction) render_time_cb, (gpointer) &tparam);
}


void render_screen(GtkWidget *widget, TRaytracer *ray)
{
  int ray_x, ray_y;
  char *ray_pixels = ray->getPixels();

  if (ray->getWidth() == 0 || ray->getHeight() == 0 ||
      widget->allocation.width == 0 || widget->allocation.height == 0)
    return;

  ray->getProgress(ray_x, ray_y);

  /* If the scaling has changed, rescale the current contents */
  if (widget->allocation.width != save_width ||
      widget->allocation.height != save_height)
  {
    save_width = widget->allocation.width;
    save_height = widget->allocation.height;

    if (copy_buffer) 
      free(copy_buffer);

    copy_buffer = (char *) malloc(3 * save_width * save_height);
    memset(copy_buffer, 0, 3 * save_width * save_height);
  
    stretch_rgb(ray_pixels, ray->getWidth(), ray->getHeight(),
                copy_buffer, save_width, save_height, 0, 0);
  }
  else if (ray_x > 0 && ray_y > 0)
  {
    /* Write all remaining pixels. */
    stretch_rgb(ray_pixels, ray->getWidth(), ray->getHeight(),
                copy_buffer, save_width, save_height, posx, posy); 
  }
  posx = stretch_wc_to_screen(ray_x, ray->getWidth(), save_width); 
  posy = stretch_wc_to_screen(ray_y, ray->getHeight(), save_height);

  /* copy buffer to server */
  gdk_draw_rgb_image(widget->window, widget->style->fg_gc[GTK_STATE_NORMAL],
                     0, 0, save_width, save_height, GDK_RGB_DITHER_NORMAL, 
                     (guchar *) copy_buffer, save_width * 3);
}

