/*
 *  This code encapsulates a scene.  Sets up an Octree for the 
 *  raytracer to use later.
 */

Scene::Scene(char *filename)
{
}

/*
 *  FIXME: Replace with a Lex/YACC parser.
 */
Scene::readFile(char *filename)
{
  #define NUMTAGS 9
  FILE *fp;
  char *Tags[]={"location:","Center:","top:","left:",
                "Image:","Lights:","Materials:","Objects:","Points:"};
  char Tag[80];
  char ColorStr[8];
  char ch;
  int i;

  char fname[MAXFNAME];     
  int inds[MAXVERTS]; int j; 
  float a, b, c, d;         
  int x,y;

  fp = fopen (filename, "r");
  if (!fp)
    return (-1);
  while (!feof(fp)) {
    fscanf (fp,"%s",Tag);     /* Look for identifier */
    for (i=0; i<NUMTAGS; i++) {
      if (!strcasecmp(Tag,Tags[i])) {
       /* Found an identifier */
       switch (i) {
         case 0:    /* Eye */
           fscanf (fp, "%f%f%f", &a, &b, &c);
           vrp[0] = a; vrp[1] = b; vrp[2] = c;
           break;
         case 1:    /* Center point */
           fscanf (fp, "%f%f%f", &a, &b, &c);
           centervp[0] = a; centervp[1] = b; centervp[2] = c;
           break;
         case 2:    /* top */
           fscanf (fp, "%f%f%f", &a, &b, &c);
           top[0]=a; top[1]=b; top[2]=c;
           break;
         case 3:    /* left */
           fscanf (fp, "%f%f%f", &a, &b, &c);
           left[0]=a; left[1]=b; left[2]=c;
           break;
         case 4:    /* Output Image */
           fscanf (fp,"%s",&fname);
           strcpy (OutputFile, fname);
           strcat (OutputFile, ".ppm");
           break;
         case 5:    /* Lights */
           fscanf (fp, "%d", &NumLights);  /* Get number of lights */
           i=0;
           while (i<NumLights) {
             fscanf (fp, "%79s", Tag);
             if (!strcasecmp(Tag,"Light:")) {
                 fscanf (fp,"%f%f%f",&a, &b, &c);
                 Lights[i].Pos = Vect(3);
                 Lights[i].Pos[0]=a; Lights[i].Pos[1]=b; Lights[i].Pos[2]=c;
                 fscanf (fp,"%8s", ColorStr);
                 CStr2RGB (ColorStr, &Lights[i].r, &Lights[i].g, &Lights[i].b);
                 fscanf (fp,"%f", &a);
                 Lights[i].in=a;
                 i++;
             }
             else if (!strcasecmp(Tag,"Lights.")) break;
             if (feof(fp)) return(0);
           }
           break;
         case 8:    /* Points */
           int NumPts;
           fscanf (fp, "%d", &NumPts);  /* Get number of lights */
           i=0;
           while (i<NumPts) {
             fscanf (fp, "%79s", Tag);
             if (!strcasecmp(Tag,"Point:")) {
               fscanf (fp,"%f%f%f",&a, &b, &c);
               Verts[3*i] = a;
               Verts[3*i+1] = b;
               Verts[3*i+2] = c;
               i++;
             }
             else if (!strcasecmp(Tag,"Points.")) break;
             if (feof(fp)) return(0);
           }
           break;
         case 6: /* Materials */
           fscanf (fp, "%d", &NumMats);
           i=0;
           while (i<NumMats) {
             fscanf (fp, "%79s", Tag);
             if (!strcasecmp(Tag, "Material:")) {
               fscanf (fp, "%8s", ColorStr);
               CStr2RGB (ColorStr, &Materials[i].dr, &Materials[i].dg, 
                         &Materials[i].db);
               fscanf (fp, "%f%f", &a, &b);
               Materials[i].ka = a;  Materials[i].kd = b;
               fscanf (fp, "%8s", ColorStr);
               CStr2RGB (ColorStr, &Materials[i].sr, &Materials[i].sg, 
                         &Materials[i].sb);
               fscanf (fp, "%f%f%f%f", &a, &b, &c, &d);
               Materials[i].ks = a; Materials[i].pow = b; 
               Materials[i].rfl = c; Materials[i].atn = d;
               fscanf (fp, "%f", &a);
               Materials[i].ir=a;
               i++;
             }
             else if (!strcasecmp (Tag, "Materials.")) break;
             if (feof(fp)) return(0);
           }
           break;
         case 7:  /* Objects */
           fscanf (fp, "%d", &NumObjs);
           i=0 ;
           while (i<NumObjs) {
             fscanf (fp, "%79s", Tag);
             if (!strcasecmp(Tag, "Sphere:")) {
               Vect ctr(3);

               fscanf (fp, "%f%f%f%f%d", &a,&b,&c,&d,&x);
               ctr[0] = b;
               ctr[1] = c;
               ctr[2] = d;
               ObjectList[i].obj = new Sphere(ctr, a); 
               ObjectList[i].Material = x;
               i++;
             }
             if (!strcasecmp (Tag, "Poly:")) {

               fscanf (fp, "%d", &x);
               for (j=0; j<x; j++) fscanf (fp, "%d", &inds[j] );
               ObjectList[i].obj = new Poly(x, inds, Verts);
               fscanf (fp, "%d", &x);
               ObjectList[i].Material = x;
               i++;
             } 
             if (!strcasecmp (Tag, "Objects.")) {
               if (i<NumObjs) NumObjs=i;
               break;
             }
             if (feof(fp)) return(0);
           }
           break;
       }
      break; 
      }   
    }
  }
  return (0);
}
