/*
 *  Simple code to stretch a buffer.  This uses no antialiasing,
 *  only big (or dropped) pixels.  The rationalization for using this
 *  is that there is no easy way to scale a Gdk (Xlib) pixmap without
 *  going to libart.  And with GdkPixbuf around the corner, writing
 *  a custom widget/canvas would duplicate effort.  So for now I scale
 *  on the client side and hand that buffer off to GdkRGB.
 */

#include <stdio.h>
#include "stretch.h"

int stretch_wc_to_screen(int x, unsigned int wc_size, unsigned int dest_size)
{
  double delta = (double) dest_size / (double) wc_size;
  return x * (int) delta;
}

void stretch_rgb(char *pixels, unsigned int width, unsigned int height, 
                 char *dest, unsigned int dest_width, 
                 unsigned int dest_height, int dest_x, int dest_y)
{
  double wdelta = (double) width / (double) dest_width;
  double hdelta = (double) height / (double) dest_height;

  /* dest_x is ignored, for now. */
  double src_x = 0;
  double src_y = hdelta * dest_y;

  int x, y, i;

  for (y = dest_y; y < dest_height; y++, src_y += hdelta)
  {
    for (x = 0, src_x = 0; x < dest_width; x++, src_x += wdelta)
    {
      for (i=0; i<3; i++)
        dest[3 * (dest_width * y + x) + i] = 
          pixels[3 * (width * (int) src_y + (int)src_x) + i];
    }
  }
}

