
#include <string.h>
#include <pthread.h>
#include "t_ray.h"

TRaytracer::TRaytracer(int width, int height) : Raytracer(width, height)
{ 
  m_status = IDLE; 
}

TRaytracer::~TRaytracer() 
{ 
  pthread_cancel(m_thread); 
  pthread_join(m_thread, NULL);
}

void TRaytracer::raytraceScene()
{
  m_status = RENDERING; 

  m_x = m_y = 0;
  memset(m_render_buf, 0, m_width * m_height * 3);  

  /* Create a new thread for the raytracer */
  pthread_create(&m_thread, NULL, (void * (*)(void *)) raytraceThread, this);
}

void TRaytracer::halt()
{
  pthread_cancel(m_thread); 
  pthread_join(m_thread, NULL);

  m_status = IDLE;
}

void TRaytracer::getProgress(int& x, int& y)
{
  x = m_x; y = m_y;
}

void TRaytracer::setWidth(int width)
{
  m_width = width;
  delete [] m_render_buf;
  m_render_buf = new char [m_width * m_height * 3];
  memset(m_render_buf, 0, m_width * m_height * 3);  
}

void TRaytracer::setHeight(int height)
{
  m_height = height;
  delete [] m_render_buf;
  m_render_buf = new char [m_width * m_height * 3];
  memset(m_render_buf, 0, m_width * m_height * 3);  
}

void TRaytracer::setStep(double step)
{
  m_step = step;
}

void TRaytracer::getProgress(double& pct)
{
  pct = (double) (m_width * m_y + m_x) / (double) (m_width * m_height); 
}

int TRaytracer::getStatus()
{
  return m_status; 
}

int TRaytracer::setStatus(int status)
{
  m_status = status; 
}

void raytraceThread(TRaytracer *R)
{
  pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
  pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
  R->Raytracer::raytraceScene();
  R->setStatus(IDLE);
}

