#include <math.h>
#include <iostream.h>
#include "misc.h"
#include "vect.h"

Vect::Vect()
{
  m_dimension = 0;
  m_v = 0;
}

Vect::Vect(int dimension)
{
  m_dimension = dimension;
  m_v = new double[m_dimension];
}

Vect::Vect(int dimension, double *values)
{
  m_dimension = dimension;
  m_v = new double[m_dimension];
  initialize(values);
}

Vect::~Vect()
{
  if (m_dimension)
  {
    delete [] m_v;
  }
}

void Vect::initialize(double *values)
{
  memcpy(m_v, values, m_dimension * sizeof(double));
}

void Vect::zero()
{
  int i;
  for (i=0; i<m_dimension; i++)
  {
    m_v[i] = 0;
  }
}

void Vect::normalize()
{
  double mag;
  int i;
 
  mag = sqrt(dot(*this)); 
  if (ABS(mag) < EPSILON) return;

  *this /= mag;
}

Vect Vect::add(const Vect &v2) const
{
  Vect out = *this;

  out += v2;
  return out;
}

Vect Vect::sub(const Vect &v2) const
{
  Vect out = *this;
  out -= v2;
  return out;
}

Vect Vect::scale(double k) const
{
  Vect out = *this;
  out *= k;
  return out;
}

double Vect::dot(const Vect &v2) const
{
  int i;
  double result = 0.0;
  for (i=0; i<m_dimension; i++)
  {
    result += m_v[i] * v2.m_v[i];
  }
  return result;
}

Vect Vect::cross(const Vect &v2) const
{
  Vect out = Vect(3);

  out.m_v[0] = m_v[1] * v2.m_v[2] - v2.m_v[1] * m_v[2];
  out.m_v[1] = - (m_v[0] * v2.m_v[2] - m_v[2] * v2.m_v[0]);
  out.m_v[2] = m_v[0] * v2.m_v[1] - m_v[1] * v2.m_v[0];

  return out;
}

int Vect::getDimension() const
{
  return m_dimension;
}

double Vect::getLength() const
{
  return sqrt(dot(*this));
}

ostream& operator<<(ostream& s, const Vect& v)
{
  int i;
  for (i=0; i<v.getDimension(); i++)
  {
    s << v[i] << ' ';
  }
  return s; 
}

Vect operator+(const Vect& lhs, const Vect& rhs)
{
  return lhs.add(rhs);
}

Vect operator-(const Vect& lhs, const Vect& rhs)
{
  return lhs.sub(rhs);
}

Vect operator*(const Vect& lhs, double rhs)
{
  return lhs.scale(rhs);
}

Vect operator*(double lhs, const Vect& rhs)
{
  return rhs.scale(lhs);
}

Vect operator/(const Vect& lhs, double rhs)
{
  return lhs.scale(1.0/rhs);
}

Vect operator/(double lhs, const Vect& rhs)
{
  return rhs.scale(1.0/lhs);
}


