#ifndef VECT_HEADER
#define VECT_HEADER

#include <iostream.h>

class Vect
{
  private:
    int m_dimension ;
    double *m_v; 

  public:
    Vect();
    Vect(int dimension);
    Vect(int dimension, double *values);
    ~Vect();
    Vect(const Vect &src);
    double& operator[](int index);
    const double& operator[](int index) const;
    Vect& operator=(const Vect& v);
    Vect& operator+=(const Vect& rhs);
    Vect& operator-=(const Vect& rhs);
    Vect& operator*=(double rhs);
    Vect& operator/=(double rhs);
    void initialize(double *values);
    void zero();
    void normalize();
    Vect add(const Vect& v2) const;
    Vect sub(const Vect& v2) const;
    Vect scale(const double k) const;
    double dot(const Vect& v2) const;
    Vect cross(const Vect& v2) const;
    int getDimension() const;
    double getLength() const;

};

ostream& operator<<(ostream& s, const Vect& v);
Vect operator+(const Vect& lhs, const Vect& rhs);
Vect operator-(const Vect& lhs, const Vect& rhs);
Vect operator*(const Vect& lhs, const double rhs);
Vect operator*(const double lhs, const Vect& rhs);
Vect operator/(const Vect& lhs, const double rhs);
Vect operator/(const double lhs, const Vect& rhs);

inline
double& Vect::operator[](int index)
{
  /* 
  should throw exception here 
  if (index > m_dimension) 
  */
  return m_v[index];
}

inline 
const double& Vect::operator[](int index) const
{
  return m_v[index];
}

inline 
Vect::Vect(const Vect &src)
{
  m_dimension = src.m_dimension;
  m_v = new double[m_dimension];
  memcpy(m_v, src.m_v, m_dimension * sizeof(double));
}

inline 
Vect& Vect::operator=(const Vect& v)
{
  if (m_dimension) delete [] m_v ;

  m_dimension = v.m_dimension;
  m_v = new double[m_dimension]; 
  memcpy(m_v, v.m_v, m_dimension * sizeof(double));
}

inline 
Vect& Vect::operator+=(const Vect& rhs)
{
  int i;
  for (i=0; i<m_dimension; i++)
  {
    m_v[i] += rhs.m_v[i];
  }
  return *this;
}

inline 
Vect& Vect::operator-=(const Vect& rhs)
{
  int i;
  for (i=0; i<m_dimension; i++)
  {
    m_v[i] -= rhs.m_v[i];
  }
  return *this;
}

inline 
Vect& Vect::operator*=(double rhs)
{
  int i;

  for (i=0; i<m_dimension; i++)
  {
    m_v[i] *= rhs;
  }
  return *this;
}

inline 
Vect& Vect::operator/=(double rhs)
{
  int i;

  for (i=0; i<m_dimension; i++)
  {
    m_v[i] /= rhs;
  }
  return *this;
}

#endif 
